function [f nCounted nUncounted]= Ediffprop( n1,n,k )

%Date: 26-May-2015
\

%Description: This code computes the result of the formula reported in Appendix E,
%Theorem 6. The formula computes the exectaed value of the difference
%between the proportion of successes on trials that are immediately
%preceded by k consecutive successes, and the proportion of successes on
%trials that are immediately preceded by k consecutive failures
%conditional on (i) there being n_1 successes and n_0 failures in the sequence, and
%(ii) there being at least one instance of a trial immediately preceded by 
%k consecutive successes and at least one instance of a trial immmediately
%preceded by k consecutive failures.

%The  output:
%   1. f: the expected difference in proportion given n1 hits in n trials.  It is the arithmetic average across all nchoosek(n,n1) sequences
%that are measurable, so we need to subtract out those sequences which are not measureable.
%   2. nUncounted: the number of sequences in which the proportion cannot be
%   calculated because the denominator is zero.  This is typically a small
%   fraction of the sequences and can be checked, e.g. for n1=25,n=50,k=3:
%       [d A B]=Ediff   prop( n1,n,k );
%       B/nchoosek(n,n1)=0.0046

% With this function for each n we can build a look up table with rows n1 and columns k% 
% we have average proportions, wait, if we have the cumSum, then we just multiply by p^n1(1-p)^(n-n1), sum
% Then divide by 1-sum_n1 nUncounted * p^n1(1-p)^(n-n1)

%Execution Notes:
% 1. With floating point arithmatic, nchoosek and like functions cease to be exact at high values
%   they remain relatively close, as can be seen when we sum the
%   probabilities and we are within 10^(-16) of the true probabilities.
%2. %for k=5 and larger the code pretty slow, perhaps the indexing isn't
%efficient?  We do have combinitorial explosion for large k, which was the
%reason this approach was taken to begin with.
%3. To check overal relative precisions of the floating point operations on the count we compute the 
%       relative error in count is:
%       abs(1-(nCounted+nUncounted)/nchoosek(n,n1)), for n=80, n1=40, k=3
%       it has 13 digits of precision.

% disp('----------------------------')
% disp('----------------------------')
%  addpath('C:\Users\Miller\Dropbox\josh\work\projects\Adam\HotHand\Data\statTheory')
%  rehash path
% %%%%%%%%%%%%%
%FOr Testing
% for m=4:9
% 
%      n=13;
%      n1=m;
%     k=4;
n0=n-n1;


if n1>=n
    error('n1 must be less than n');
    return,
end

if k>n1
    error('k must be less than n1');
    return,
end

if k>n0
    error('k must be less than n0');
    return,
end

TseqNotCounted=0;
cumdiffprop=0;
TseqCounted=0;
nseq=0;
multiplier=0;






%initialize variables that will be over-written repeatedly
f1k=0;
f0k=0;
m1k=0;
m0k=0;
diffstat=0;
FixedLength1runs=zeros(1,k-1);
r1=0;
FixedLength0runs=zeros(1,k-1);
r0=0;
multiplier=0;
Nsequences=0;

if k==2
    for s1k=0:floor( n1/2 )
    for r11=0:floor( (n1-2*s1k)/1 )
        FixedLength1runs=[r11]; %Instead of 1:floor(n1/j)+1 it is 0:floor(n1/j)
        r1=sum([FixedLength1runs s1k]); %total  1runs

        for s0k=0:min(floor( n0/2 ), r1+1)
        %FOLDED CODE JUST BELOW, IT IS IDENTICAL FOR ALL LOOPS except first
        %line
        for r01=0:min(floor( (n0-2*s0k)/1 ), r1+1)

            FixedLength0runs=[r01];
            r0=sum([FixedLength0runs s0k]); %total 0 runs

            n1sInFixedLength1runs=sum( ( 1:length(FixedLength1runs) ).*FixedLength1runs );
            n0sInFixedLength0runs=sum( ( 1:length(FixedLength0runs) ).*FixedLength0runs );  
            if  abs(r1-r0)<=1

                Nsequences=NseqsRunsOfTwoTypes( FixedLength1runs,s1k,n1, FixedLength0runs,s0k, n0);
                multiplier=2*(r1==r0)+( abs(r1-r0)==1);
                nseq=multiplier*Nsequences;


                if (s1k>=1&&s0k>=1 &&  n1sInFixedLength1runs<=n1-k && n0sInFixedLength0runs <=n0-k)

                    %TseqCounted=TseqCounted+ nseq;

                    f1k = freqstat(FixedLength1runs,s1k,n1);
                    f0k = freqstat(FixedLength0runs,s0k,n0);

                    if (r0==r1+1) %Must end with a run of 0s. We are within Case 1 or Case 3i
                        %ADD UP ALL CASES HERE

                        %case 1: ends in streak of 0s of k or more
                        if n0sInFixedLength0runs==n0-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s0k/r0 )* Nsequences;
                        elseif n0sInFixedLength0runs<n0-k
                            TseqCounted=TseqCounted+ (s0k/r0 )* Nsequences;
                            m1k = (f1k-s1k)/f1k;  %because ends with a run of 0s
                            m0k = (f0k-s0k)/(f0k-1);  %because ends with run of 0s of k or more
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s0k/r0 )* Nsequences;


                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %Case 3i: ends in streak of 0s with less than k
                        TseqCounted=TseqCounted+((r0-s0k)/r0 )*Nsequences;
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 0s
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 0s of k-1 or less
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r0-s0k)/r0 )*Nsequences;





                    elseif (r0==r1) %May end with run of 1s or run of 0s Within Case 1, Case 2, or Case 3i, case 3ii
                        %ADD UP ALL CASES HERE
                        %case 1: ends in streak of 0s of k or more
                        if n0sInFixedLength0runs==n0-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s0k/r0 )* Nsequences;
                        elseif n0sInFixedLength0runs<n0-k
                            TseqCounted=TseqCounted+(s0k/r0 )* Nsequences;
                            m1k = (f1k-s1k)/f1k;  %because ends with a run of 0s
                            m0k = (f0k-s0k)/(f0k-1);  %because ends with run of 0s of k or more
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s0k/r0 )* Nsequences;

                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %case 2: ends in streak of 1s of k or more
                        if n1sInFixedLength1runs==n1-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s1k/r1 )* Nsequences;
                        elseif n1sInFixedLength1runs<n1-k
                            TseqCounted=TseqCounted+(s1k/r1 )* Nsequences;
                            m1k = (f1k-s1k)/(f1k-1);  %because ends with a run of 1s of k or more
                            m0k = (f0k-s0k)/f0k;  %because ends with run of 1s 
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s1k/r1 )* Nsequences;


                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %Case 3i: ends in streak of 0s with less than k
                        TseqCounted=TseqCounted+((r0-s0k)/r0 )* Nsequences;
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 0s
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 0s of length k-1 or less
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r0-s0k)/r0 )* Nsequences;


                        %Case 3ii: ends in streak of 1s with less than k
                        TseqCounted=TseqCounted+((r1-s1k)/r1 )* Nsequences; 
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 1s of length k-1 or less
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 1s
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r1-s1k)/r1 )* Nsequences; 




                    elseif (r0==r1-1) %Must end with a run of 1s. Within Case 2 and case 3ii
                        %ADD UP ALL CASES HERE
                        %case 2: ends in streak of 1s of k or more
                        if n1sInFixedLength1runs==n1-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s1k/r1 )* Nsequences;
                        elseif n1sInFixedLength1runs<n1-k
                            TseqCounted=TseqCounted+(s1k/r1 )* Nsequences;
                            m1k = (f1k-s1k)/(f1k-1);  %because ends with a run of 1s of k or more
                            m0k = (f0k-s0k)/f0k;  %because ends with run of 1s 
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s1k/r1 )* Nsequences;


                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %Case 3ii: ends in streak of 1s with less than k
                        TseqCounted=TseqCounted+((r1-s1k)/r1 )* Nsequences;
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 1s of length k-1 or less
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 1s
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r1-s1k)/r1 )* Nsequences;



                    else
                               error('Problem with control flow');
                               return, 
                    end


                else
                   TseqNotCounted= TseqNotCounted +  nseq;
                end
            end

        end
        end
    end
    end 
    
elseif k==3
    for s1k=0:floor( n1/3 )
    for r12=0:floor( (n1-3*s1k)/2 )
    for r11=0:floor( (n1-3*s1k-2*r12)/1 )
        FixedLength1runs=[r11 r12]; %Instead of 1:floor(n1/j)+1 it is 0:floor(n1/j)
        r1=sum([FixedLength1runs s1k]); %total  1runs

        for s0k=0:min(floor( n0/3 ), r1+1)
        for r02=0:min(floor( (n0-3*s0k)/2 ), r1+1)
        %FOLDED CODE JUST BELOW, IT IS IDENTICAL FOR ALL LOOPS except first
        %line
        for r01=0:min(floor( (n0-3*s0k-2*r02)/1 ), r1+1)

            FixedLength0runs=[r01 r02];
            r0=sum([FixedLength0runs s0k]); %total 0 runs

            n1sInFixedLength1runs=sum( ( 1:length(FixedLength1runs) ).*FixedLength1runs );
            n0sInFixedLength0runs=sum( ( 1:length(FixedLength0runs) ).*FixedLength0runs );  
            if  abs(r1-r0)<=1

                Nsequences=NseqsRunsOfTwoTypes( FixedLength1runs,s1k,n1, FixedLength0runs,s0k, n0);
                multiplier=2*(r1==r0)+( abs(r1-r0)==1);
                nseq=multiplier*Nsequences;


                if (s1k>=1&&s0k>=1 &&  n1sInFixedLength1runs<=n1-k && n0sInFixedLength0runs <=n0-k)

                    %TseqCounted=TseqCounted+ nseq;

                    f1k = freqstat(FixedLength1runs,s1k,n1);
                    f0k = freqstat(FixedLength0runs,s0k,n0);

                    if (r0==r1+1) %Must end with a run of 0s. We are within Case 1 or Case 3i
                        %ADD UP ALL CASES HERE

                        %case 1: ends in streak of 0s of k or more
                        if n0sInFixedLength0runs==n0-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s0k/r0 )* Nsequences;
                        elseif n0sInFixedLength0runs<n0-k
                            TseqCounted=TseqCounted+ (s0k/r0 )* Nsequences;
                            m1k = (f1k-s1k)/f1k;  %because ends with a run of 0s
                            m0k = (f0k-s0k)/(f0k-1);  %because ends with run of 0s of k or more
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s0k/r0 )* Nsequences;


                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %Case 3i: ends in streak of 0s with less than k
                        TseqCounted=TseqCounted+((r0-s0k)/r0 )*Nsequences;
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 0s
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 0s of k-1 or less
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r0-s0k)/r0 )*Nsequences;





                    elseif (r0==r1) %May end with run of 1s or run of 0s Within Case 1, Case 2, or Case 3i, case 3ii
                        %ADD UP ALL CASES HERE
                        %case 1: ends in streak of 0s of k or more
                        if n0sInFixedLength0runs==n0-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s0k/r0 )* Nsequences;
                        elseif n0sInFixedLength0runs<n0-k
                            TseqCounted=TseqCounted+(s0k/r0 )* Nsequences;
                            m1k = (f1k-s1k)/f1k;  %because ends with a run of 0s
                            m0k = (f0k-s0k)/(f0k-1);  %because ends with run of 0s of k or more
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s0k/r0 )* Nsequences;

                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %case 2: ends in streak of 1s of k or more
                        if n1sInFixedLength1runs==n1-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s1k/r1 )* Nsequences;
                        elseif n1sInFixedLength1runs<n1-k
                            TseqCounted=TseqCounted+(s1k/r1 )* Nsequences;
                            m1k = (f1k-s1k)/(f1k-1);  %because ends with a run of 1s of k or more
                            m0k = (f0k-s0k)/f0k;  %because ends with run of 1s 
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s1k/r1 )* Nsequences;


                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %Case 3i: ends in streak of 0s with less than k
                        TseqCounted=TseqCounted+((r0-s0k)/r0 )* Nsequences;
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 0s
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 0s of length k-1 or less
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r0-s0k)/r0 )* Nsequences;


                        %Case 3ii: ends in streak of 1s with less than k
                        TseqCounted=TseqCounted+((r1-s1k)/r1 )* Nsequences; 
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 1s of length k-1 or less
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 1s
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r1-s1k)/r1 )* Nsequences; 




                    elseif (r0==r1-1) %Must end with a run of 1s. Within Case 2 and case 3ii
                        %ADD UP ALL CASES HERE
                        %case 2: ends in streak of 1s of k or more
                        if n1sInFixedLength1runs==n1-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s1k/r1 )* Nsequences;
                        elseif n1sInFixedLength1runs<n1-k
                            TseqCounted=TseqCounted+(s1k/r1 )* Nsequences;
                            m1k = (f1k-s1k)/(f1k-1);  %because ends with a run of 1s of k or more
                            m0k = (f0k-s0k)/f0k;  %because ends with run of 1s 
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s1k/r1 )* Nsequences;


                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %Case 3ii: ends in streak of 1s with less than k
                        TseqCounted=TseqCounted+((r1-s1k)/r1 )* Nsequences;
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 1s of length k-1 or less
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 1s
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r1-s1k)/r1 )* Nsequences;



                    else
                               error('Problem with control flow');
                               return, 
                    end


                else
                   TseqNotCounted= TseqNotCounted +  nseq;
                end
            end

        end
        end
        end
    end
    end
    end

elseif k==4
       for s1k=0:floor( n1/4 )
       for r13=0:floor( (n1-4*s1k)/3 )
       for r12=0:floor( (n1-4*s1k-3*r13)/2 )
       for r11=0:floor( (n1-4*s1k-3*r13-2*r12)/1 )
        FixedLength1runs=[r11 r12 r13]; %Instead of 1:floor(n1/j)+1 it is 0:floor(n1/j)
        r1=sum([FixedLength1runs s1k]); %total  1runs

        for s0k=0:min(floor( n0/4 ), r1+1)
        for r03=0:min(floor( (n0-4*s0k)/3 ), r1+1)
        for r02=0:min(floor( (n0-4*s0k-3*r03)/2 ), r1+1)
        %FOLDED CODE JUST BELOW, IT IS IDENTICAL FOR ALL LOOPS except first
        %line
         for r01=0:min(floor( (n0-4*s0k-3*r03-2*r02)/1 ), r1+1)

            FixedLength0runs=[r01 r02 r03];
            r0=sum([FixedLength0runs s0k]); %total 0 runs

            n1sInFixedLength1runs=sum( ( 1:length(FixedLength1runs) ).*FixedLength1runs );
            n0sInFixedLength0runs=sum( ( 1:length(FixedLength0runs) ).*FixedLength0runs );  
            if  abs(r1-r0)<=1

                Nsequences=NseqsRunsOfTwoTypes( FixedLength1runs,s1k,n1, FixedLength0runs,s0k, n0);
                multiplier=2*(r1==r0)+( abs(r1-r0)==1);
                nseq=multiplier*Nsequences;


                if (s1k>=1&&s0k>=1 &&  n1sInFixedLength1runs<=n1-k && n0sInFixedLength0runs <=n0-k)

                    %TseqCounted=TseqCounted+ nseq;

                    f1k = freqstat(FixedLength1runs,s1k,n1);
                    f0k = freqstat(FixedLength0runs,s0k,n0);

                    if (r0==r1+1) %Must end with a run of 0s. We are within Case 1 or Case 3i
                        %ADD UP ALL CASES HERE

                        %case 1: ends in streak of 0s of k or more
                        if n0sInFixedLength0runs==n0-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s0k/r0 )* Nsequences;
                        elseif n0sInFixedLength0runs<n0-k
                            TseqCounted=TseqCounted+ (s0k/r0 )* Nsequences;
                            m1k = (f1k-s1k)/f1k;  %because ends with a run of 0s
                            m0k = (f0k-s0k)/(f0k-1);  %because ends with run of 0s of k or more
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s0k/r0 )* Nsequences;


                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %Case 3i: ends in streak of 0s with less than k
                        TseqCounted=TseqCounted+((r0-s0k)/r0 )*Nsequences;
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 0s
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 0s of k-1 or less
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r0-s0k)/r0 )*Nsequences;





                    elseif (r0==r1) %May end with run of 1s or run of 0s Within Case 1, Case 2, or Case 3i, case 3ii
                        %ADD UP ALL CASES HERE
                        %case 1: ends in streak of 0s of k or more
                        if n0sInFixedLength0runs==n0-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s0k/r0 )* Nsequences;
                        elseif n0sInFixedLength0runs<n0-k
                            TseqCounted=TseqCounted+(s0k/r0 )* Nsequences;
                            m1k = (f1k-s1k)/f1k;  %because ends with a run of 0s
                            m0k = (f0k-s0k)/(f0k-1);  %because ends with run of 0s of k or more
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s0k/r0 )* Nsequences;

                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %case 2: ends in streak of 1s of k or more
                        if n1sInFixedLength1runs==n1-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s1k/r1 )* Nsequences;
                        elseif n1sInFixedLength1runs<n1-k
                            TseqCounted=TseqCounted+(s1k/r1 )* Nsequences;
                            m1k = (f1k-s1k)/(f1k-1);  %because ends with a run of 1s of k or more
                            m0k = (f0k-s0k)/f0k;  %because ends with run of 1s 
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s1k/r1 )* Nsequences;


                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %Case 3i: ends in streak of 0s with less than k
                        TseqCounted=TseqCounted+((r0-s0k)/r0 )* Nsequences;
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 0s
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 0s of length k-1 or less
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r0-s0k)/r0 )* Nsequences;


                        %Case 3ii: ends in streak of 1s with less than k
                        TseqCounted=TseqCounted+((r1-s1k)/r1 )* Nsequences; 
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 1s of length k-1 or less
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 1s
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r1-s1k)/r1 )* Nsequences; 




                    elseif (r0==r1-1) %Must end with a run of 1s. Within Case 2 and case 3ii
                        %ADD UP ALL CASES HERE
                        %case 2: ends in streak of 1s of k or more
                        if n1sInFixedLength1runs==n1-k % if it is exactly length k it is not measurable
                            TseqNotCounted= TseqNotCounted + (s1k/r1 )* Nsequences;
                        elseif n1sInFixedLength1runs<n1-k
                            TseqCounted=TseqCounted+(s1k/r1 )* Nsequences;
                            m1k = (f1k-s1k)/(f1k-1);  %because ends with a run of 1s of k or more
                            m0k = (f0k-s0k)/f0k;  %because ends with run of 1s 
                            diffstat=m1k-(1-m0k);

                            cumdiffprop = cumdiffprop + diffstat*(s1k/r1 )* Nsequences;


                        else
                               error('Problem with control flow');
                               return,                        
                        end

                        %Case 3ii: ends in streak of 1s with less than k
                        TseqCounted=TseqCounted+((r1-s1k)/r1 )* Nsequences;
                        m1k = (f1k-s1k)/f1k;  %because ends with run of 1s of length k-1 or less
                        m0k = (f0k-s0k)/f0k;  %because ends with run of 1s
                        diffstat=m1k-(1-m0k);
                        cumdiffprop = cumdiffprop + diffstat*((r1-s1k)/r1 )* Nsequences;



                    else
                               error('Problem with control flow');
                               return, 
                    end


                else
                   TseqNotCounted= TseqNotCounted +  nseq;
                end
            end

        end
        end
        end
        end
    end
    end
    end
    end
    
else
    error('Code only handles k=2,3,4');
                               return,  
   %Write code if necessary, repeat pattern before.
   %We cannot linearize the nested indices because we need to maintain
   %dependence to reduce the number of terms in the sum.
end

nCounted=TseqCounted;
nUncounted=TseqNotCounted;
f=cumdiffprop/TseqCounted;

% 
% 
% % TESTING
% % Below output for testing, to make sure things add correctly
% % Confirm the partitition of the r11,r12,s13 sequences, that total count, divided by
% % nchoosek(n,n1), sums to 1.
% 
% 
% disp('-----------------------------')
% disp('************  Final Count ****************')
% disp('-----------------------------')
% disp('n1:')
% n1
% 
% disp('Counted Sequences:')
% TseqCounted
% 
% disp('UnCounted Sequences:')
% TseqNotCounted
% 
% disp('Total Sequences:')
% mycount=TseqCounted+TseqNotCounted
% 
% disp('True Count')
% disp('Combination Function')
% truecount=nchoosek(n,n1)
% 
% 
% disp('Absolute Difference')
% diff=abs(truecount-mycount)
% 
% disp('Relative Difference: This is the key check') 
% pdiff=abs(1-mycount/truecount)
% 
% 
% disp('Expected Difference in proportion')
% cumdiffprop/TseqCounted
% 
% end





end


%ANOTHER WAY TO COUNT THE UNDEFINED: BINOMIAL EXPANSION FROM PAPER

%%TEST SUM DIFF
% clear
% n=10;
% n1=5;
% k=3;
% n0=n-n1;
% 
% [f nCounted nUncounted]= Ediffprop( n1,n,k )
% 
% 
% nseq1i=0;
% for r1=1:min(n1,n0+1)
%     ulim=min( floor( (n1-r1)/(k-1) ), r1);
%     nmult=0;
%     for t1=0:ulim
%         t2=n1-r1-t1*(k-1);
%         nmult=nmult+ (-1)^t1*nchoosek(r1,t1)*nchoosek(r1-1+t2,r1-1);
%     end
%     nseq1i=nseq1i+nmult*nchoosek(n0+1,r1);
%     
% end
% 
% disp(' ****** Case 1ii ****** ')
% %Case 1ii
% %Here we count the number of sequences that end in a run of k, with all
% %other runs less than k length.
% nseq1ii=0;
% for r1=2:min(n1,n0+1)
%     ulim=min( floor( (n1-k-(r1-1))/(k-1) ), r1-1);
%     nmult=0;
%     for t1=0:ulim
%         t2=n1-k-(r1-1)-t1*(k-1);
%         nmult=nmult+ (-1)^t1*nchoosek(r1-1,t1)*nchoosek(r1-2+t2,r1-2);
%     end
%     nseq1ii=nseq1ii+nmult*nchoosek(n0,r1-1);
%     
% end
% 
% nseq2i=0;
% for r0=1:min(n0,n1+1)
%     ulim=min( floor( (n0-r0)/(k-1) ), r0);
%     nmult=0;
%     for t1=0:ulim
%         t2=n0-r0-t1*(k-1);
%         nmult=nmult+ (-1)^t1*nchoosek(r0,t1)*nchoosek(r0-1+t2,r0-1);
%     end
%     nseq2i=nseq2i+nmult*nchoosek(n1+1,r0);
%     
% end
% 
% disp(' ****** Case 1ii ****** ')
% %Case 1ii
% %Here we count the number of sequences that end in a run of k, with all
% %other runs less than k length.
% nseq2ii=0;
% for r0=2:min(n0,n1+1)
%     ulim=min( floor( (n0-k-(r0-1))/(k-1) ), r0-1);
%     nmult=0;
%     for t1=0:ulim
%         t2=n0-k-(r0-1)-t1*(k-1);
%         nmult=nmult+ (-1)^t1*nchoosek(r0-1,t1)*nchoosek(r0-2+t2,r0-2);
%     end
%     nseq2ii=nseq2ii+nmult*nchoosek(n1,r0-1);
%     
% end
% 
% nseq3=0;
% for r0=2:min(n0,n1+1)
%     for r1=max(r0-1,1):min(r0+1,n1)
%         ulim=min( floor( (n0-r0)/(k-1) ), r0);
%         nmult1=0;
%         for t1=0:ulim
%             t2=n0-r0-t1*(k-1);
%             nmult1=nmult1+ (-1)^t1*nchoosek(r0,t1)*nchoosek(r0-1+t2,r0-1);
%         end
% 
%         ulim=min( floor( (n1-r1)/(k-1) ), r1);
%         nmult2=0;
%         for t1=0:ulim
%             t2=n1-r1-t1*(k-1);
%             nmult2=nmult2+ (-1)^t1*nchoosek(r1,t1)*nchoosek(r1-1+t2,r1-1);
%         end
%         nseq3=nseq3+(2*[r1==r0]+1*[abs(r1-r0)==1])*nmult1*nmult2;
%     end
% end
% 
% 
% nseq4i=0;
% for r0=2:min(n0,n1+1)
%     for r1=max(r0-1,1):min(r0,n1)
%         ulim=min( floor( (n0-k-(r0-1))/(k-1) ), r0-1);
%         nmult1=0;
%         for t1=0:ulim
%             t2=n0-k-(r0-1)-t1*(k-1);
%             nmult1=nmult1+ (-1)^t1*nchoosek(r0-1,t1)*nchoosek(r0-2+t2,r0-2);
%         end
% 
%         ulim=min( floor( (n1-r1)/(k-1) ), r1);
%         nmult2=0;
%         for t1=0:ulim
%             t2=n1-r1-t1*(k-1);
%             nmult2=nmult2+ (-1)^t1*nchoosek(r1,t1)*nchoosek(r1-1+t2,r1-1);
%         end
%         nseq4i=nseq4i+nmult1*nmult2;
%     end
% end
% 
% nseq4ii=0;
% for r1=2:min(n1,n0+1)
%     for r0=max(r1-1,1):min(r1,n0)
%         ulim=min( floor( (n1-k-(r1-1))/(k-1) ), r1-1);
%         nmult1=0;
%         for t1=0:ulim
%             t2=n1-k-(r1-1)-t1*(k-1);
%             nmult1=nmult1+ (-1)^t1*nchoosek(r1-1,t1)*nchoosek(r1-2+t2,r1-2);
%         end
% 
%         ulim=min( floor( (n0-r0)/(k-1) ), r0);
%         nmult2=0;
%         for t1=0:ulim
%             t2=n0-r0-t1*(k-1);
%             nmult2=nmult2+ (-1)^t1*nchoosek(r0,t1)*nchoosek(r0-1+t2,r0-1);
%         end
%         nseq4ii=nseq4ii+nmult1*nmult2;
%     end
% end
% 
% 
% Total=nseq1i+nseq1ii+nseq2i+nseq2ii-nseq3-nseq4i-nseq4ii