#!/usr/bin/env python

"""diffP.py: 
    Builds dictionary that can be used to 
    1. Create exact sampling distribution of the difference in proportions.
    2. Calculate the expected difference in proportions, 
    explanation below
"""





#Explanation:
#Consider the difference in proportion of hits after 1 hit vs. 1 miss
#Each sequence has a tuple (#miss after miss, #hit after miss, #miss after hit, #hit after hit) 
#Because tuples are immutable, they can serve as keys in a dictionary.

#With 3 shots we have the following possibilities.
#sequence, dictionary = {key=(#miss after hit, #hit after hit): value=# sequences}
# 000       {(2,0,0,0):q^3}
# 001       {(1,1,0,0):q^2p}
# 010       {(0,1,1,0):q^2p}
# 100       {(1,0,1,0):q^2p}
# 110       {(0,0,1,1):qp^2}
# 101       {(0,1,1,0):qp^2}
# 011       {(0,1,0,1):qp^2}
# 111       {(0,0,0,2):p^3}

# we can add these dictionaries together to get:
#     B[0][0][3]= {(0,1,1,0):q^2p,(2,0,0,0):q^3,(1,1,0,0):q^2p,(1,0,1,0):q^2p,(0,0,1,1):qp^2,(0,1,0,1):qp^2,(0,0,0,2):p^3 }
# were for B[m][h][c], 
# h=# of consecutive misses to the left
# h=# of consecutive hits to the left
# c=# of remaining shots to the right

# With this dictionary, it is trivial, and quick to:
# 1. Compute expected proportion (function defined below) 
# 2. Plot a histogram of its values (i.e. the sampling distribution)

# The trick then to define a recursion to build this dictionary
# The function outcome_and_frequency_dictionary below does this by
# initializing a few dictionaries, and then building on them,
# for each additional shot that is added, hit or miss.




#Use Counter dictionary subclass
#Reason: when combining 2 dicts, items with matching keys add their values.
from collections import Counter 

import gc  #garbage collector for memeory management.

#With the dictionary dict_on_a_streak consisting of sequences
#in which adding the next outcome will create a hit after a streak of hits
#or a miss after a streaks of hits, then we need to increase the count of each

 
def update_count_and_probability(miss_streak,hit_streak,hit,prob_hit,dict_on_a_streak):
    p=prob_hit
    q=1-p
    updated_dict=Counter({})
    for key, value in dict_on_a_streak.items():
        if hit_streak:
            return Counter({(key[0], key[1],key[2] + 1-hit, key[3] + hit):value*p**hit*q**(1-hit) \
                for key, value in dict_on_a_streak.items()})
        elif miss_streak:
            return Counter({(key[0]+1-hit, key[1]+hit,key[2],key[3] ):value*p**hit*q**(1-hit) \
                for key, value in dict_on_a_streak.items()})
        else:
            return Counter({(key[0], key[1],key[2],key[3]):value*p**hit*q**(1-hit) \
                for key, value in dict_on_a_streak.items()})


    
# Building the main dictionary                   
def outcome_and_frequency_dictionary(number_of_shots,streak_length,probability_of_hit):
    miss_streak, hit_streak, shots_right   = number_of_shots+1, number_of_shots+1, number_of_shots+1; 
    p=probability_of_hit
    q=1-p
    
    #Initiliaze matrix of empty dictionaries rows x col
    B = [[[Counter({}) for m in range(miss_streak)] for h in range(hit_streak)] for s in range(shots_right)]

    # Initialize the recursion
    # If there is as streak of r hits on the left, and no more shots to the right
    # Then there is just one way for this to happen
    for m in range(miss_streak):
         B[m][0][0]=Counter({(0,0,0,0):1})
    for h in range(hit_streak):
         B[0][h][0]=Counter({(0,0,0,0):1})

        #HERERE*************************************
    #update_count_and_probability(miss_streak,hit_streak,hit,prob_hit,dict_on_a_streak)
    for n in range(1,number_of_shots+1):
        k=streak_length
        if n<=k:
            for i in  range(n-1, 0, -1):
                B[i][0][n-i] = update_count_and_probability(0,0,0,p,B[i+1][0][n-i-1]) \
                    + update_count_and_probability(0,0,1,p,B[0][1][n-i-1])
                
                B[0][i][n-i] = update_count_and_probability(0,0,0,p,B[1][0][n-i-1]) \
                    + update_count_and_probability(0,0,1,p,B[0][i+1][n-i-1])
                    
            B[0][0][n] = update_count_and_probability(0,0,0,p,B[1][0][n-1]) \
                + update_count_and_probability(0,0,1,p,B[0][1][n-1])
        if n>k:
            #case in which add a streak successor
            B[k][0][n-k] = update_count_and_probability(1,0,0,p,B[k][0][n-k-1]) \
                + update_count_and_probability(1,0,1,p,B[0][1][n-k-1])
                
            B[0][k][n-k] = update_count_and_probability(0,1,0,p,B[1][0][n-k-1]) \
                + update_count_and_probability(0,1,1,p,B[0][k][n-k-1])  
                
            #case in which do not add streak successor
            for i in  range(k-1, 0, -1):
                B[i][0][n-i] = update_count_and_probability(0,0,0,p,B[i+1][0][n-i-1]) \
                + update_count_and_probability(0,0,1,p,B[0][1][n-i-1])
                
                B[0][i][n-i] = update_count_and_probability(0,0,0,p,B[1][0][n-i-1]) \
                + update_count_and_probability(0,0,1,p,B[0][i+1][n-i-1])
           
            B[0][0][n] = update_count_and_probability(0,0,0,p,B[1][0][n-1]) \
                + update_count_and_probability(0,0,1,p,B[0][1][n-1])
    return B

#This should be self explantory.
def expected_difference(number_of_shots,streak_length,B):    
    num=0
    den=0
    for number_of_events_of_each_kind, probability \
        in B[0][0][number_of_shots].items():
        
        if number_of_events_of_each_kind[0:2]==(0,0) or number_of_events_of_each_kind[2:4]==(0,0):continue
        
        number_of_hits_after_streak_of_misses=number_of_events_of_each_kind[1]
        number_of_hits_after_streak_of_hits=number_of_events_of_each_kind[3]
        
        
        number_of_shots_after_streak_of_misses= \
            number_of_events_of_each_kind[0]+number_of_events_of_each_kind[1]
        number_of_shots_after_streak_of_hits= \
            number_of_events_of_each_kind[2]+number_of_events_of_each_kind[3]
            
        
        proportion_hits_after_streak_of_misses= \
            number_of_hits_after_streak_of_misses/number_of_shots_after_streak_of_misses    
        proportion_hits_after_streak_of_hits= \
            number_of_hits_after_streak_of_hits/number_of_shots_after_streak_of_hits
        
        difference=proportion_hits_after_streak_of_hits-proportion_hits_after_streak_of_misses
        
        #print(difference,probability)
        num += probability*difference
        den += probability
        #print(num,den)
    return num/den


def probability_difference_undefined(number_of_shots,streak_length,B):    
    probability_missing = 0

    for number_of_events_of_each_kind, probability \
        in B[0][0][number_of_shots].items():
        
        if number_of_events_of_each_kind[0:2]==(0,0) or number_of_events_of_each_kind[2:4]==(0,0):
            probability_missing += probability
        
    return probability_missing


#Modifed version of the code to handle total hits
def update_count_and_probability_totalhits(miss_streak,hit_streak,hit,prob_hit,dict_on_a_streak):
    p=prob_hit
    q=1-p
    updated_dict=Counter({})
    for key, value in dict_on_a_streak.items():
        if hit_streak:
            return Counter({(key[0], key[1],key[2] + 1-hit, key[3] + hit, key[4] + hit):value*p**hit*q**(1-hit) \
                for key, value in dict_on_a_streak.items()})
        elif miss_streak:
            return Counter({(key[0]+1-hit, key[1]+hit,key[2],key[3], key[4] + hit ):value*p**hit*q**(1-hit) \
                for key, value in dict_on_a_streak.items()})
        else:
            return Counter({(key[0], key[1],key[2],key[3], key[4] + hit):value*p**hit*q**(1-hit) \
                for key, value in dict_on_a_streak.items()})


    
# Building the main dictionary                   
def outcome_and_frequency_dictionary_totalhits(number_of_shots,streak_length,probability_of_hit):
    miss_streak, hit_streak, shots_right   = streak_length+1, streak_length+1, number_of_shots+1; 
    p=probability_of_hit
    q=1-p
    
    #Initiliaze dicionary 
    B = {}

    # Initialize the recursion
    # If there is as streak of r hits on the left, and no more shots to the right
    # Then there is just one way for this to happen
    # The keys are tuples, the values are Counter Constructs
    for m in range(miss_streak):
         B[m,0,0]=Counter({(0,0,0,0,0):1})
    for h in range(hit_streak):
         B[0,h,0]=Counter({(0,0,0,0,0):1})

        #HERERE*************************************
    #update_count_and_probability_totalhits(miss_streak,hit_streak,hit,prob_hit,dict_on_a_streak)
    for n in range(1,number_of_shots+1):
        k=streak_length
        if n<=k:
            for i in  range(n-1, 0, -1):
                B[(i,0,n-i)] = update_count_and_probability_totalhits(0,0,0,p,B[i+1,0,n-i-1]) \
                    + update_count_and_probability_totalhits(0,0,1,p,B[0,1,n-i-1])
                
                B[0,i,n-i] = update_count_and_probability_totalhits(0,0,0,p,B[1,0,n-i-1]) \
                    + update_count_and_probability_totalhits(0,0,1,p,B[0,i+1,n-i-1])
                    
            B[0,0,n] = update_count_and_probability_totalhits(0,0,0,p,B[1,0,n-1]) \
                + update_count_and_probability_totalhits(0,0,1,p,B[0,1,n-1])
        if n>k:
            #case in which add a streak successor
            B[k,0,n-k] = update_count_and_probability_totalhits(1,0,0,p,B[k,0,n-k-1]) \
                + update_count_and_probability_totalhits(1,0,1,p,B[0,1,n-k-1])
                
            B[0,k,n-k] = update_count_and_probability_totalhits(0,1,0,p,B[1,0,n-k-1]) \
                + update_count_and_probability_totalhits(0,1,1,p,B[0,k,n-k-1])  
                
            #case in which do not add streak successor
            for i in  range(k-1, 0, -1):
                B[i,0,n-i] = update_count_and_probability_totalhits(0,0,0,p,B[i+1,0,n-i-1]) \
                + update_count_and_probability_totalhits(0,0,1,p,B[0,1,n-i-1])
                
                B[0,i,n-i] = update_count_and_probability_totalhits(0,0,0,p,B[1,0,n-i-1]) \
                + update_count_and_probability_totalhits(0,0,1,p,B[0,i+1,n-i-1])
           
            B[0,0,n] = update_count_and_probability_totalhits(0,0,0,p,B[1,0,n-1]) \
                + update_count_and_probability_totalhits(0,0,1,p,B[0,1,n-1])
            
            if n>k:
                del B[0,0,n-k-1]
                for m in range(1,miss_streak):
                    del B[m,0,n-k-1]
                for h in range(1,hit_streak):
                    del B[0,h,n-k-1]
             
            gc.collect()
            print("Step ",n)
    return B

    # Build a dictionary where the items defined unique proprotions
# key = proportion of hits after streak_length hits
# value = probability of occurance
def histogram_counts_totalhits(number_of_shots,number_of_hits,streak_length,B): 
    histogram = Counter({})           
    for number_of_events_of_each_kind, probability \
        in B[0,0,number_of_shots].items():
        
        if number_of_events_of_each_kind[0:2]==(0,0) or number_of_events_of_each_kind[2:4]==(0,0) or number_of_events_of_each_kind[4]!=number_of_hits:continue
        
            
        number_of_hits_after_streak_of_hits=number_of_events_of_each_kind[3]
        
        number_of_shots_after_streak_of_hits= \
            number_of_events_of_each_kind[2]+number_of_events_of_each_kind[3]
            
        proportion_hits_after_streak_of_hits= \
            number_of_hits_after_streak_of_hits/number_of_shots_after_streak_of_hits
        
        
        number_of_hits_after_streak_of_misses=number_of_events_of_each_kind[1]
        
        number_of_shots_after_streak_of_misses= \
            number_of_events_of_each_kind[0]+number_of_events_of_each_kind[1]
            
        proportion_hits_after_streak_of_misses= \
            number_of_hits_after_streak_of_misses/number_of_shots_after_streak_of_misses

        difference = proportion_hits_after_streak_of_hits - proportion_hits_after_streak_of_misses
        

            
        histogram = histogram + Counter({difference:probability})
    return histogram