*These programs appear in Analysis_Surprised

cap program drop genshots_regimeswitch
program define genshots_regimeswitch
	syntax [, staynormal(real .95) frachot(real .2) nshots(integer 100) fg(real .5) dfg(real .10) ]
	*Note: If we want to keep the long run frequency of the hot hand state constant
	* 	and increase the likliness of staying in the hot state, then we have to make 
	*   the chance of trainsitioning into the hot state lower, we have to wait
	* 	longer to see a hot state nbinomial(1, 70,.01)
	* For the tranistion matrix P, let p=Pr(0|0), q=Pr(1|1)
	* The stationary pi distrubution statisfied pi0/pi1=(1-q)/(1-p)
	scalar drop _all
	scalar fg=`fg'  //Overall probability of a hit
	scalar dfg=`dfg'
	scalar pi=`frachot'
	scalar fgn=scalar(fg)-scalar(pi)*scalar(dfg) // normal state probability chosen so that they average to .5
	scalar fgh=scalar(fgn)+scalar(dfg)
	display "fgh = " fgh ", fgn = " fgn ", fg= " fg ", frachot= " pi
	assert scalar(fgn)<=1& scalar(fgn)>=0
	assert scalar(fgh)<=1& scalar(fgh)>=0
	scalar p=`staynormal'
	*from the stationary distrubtion formula pi=pi*P, we calculate the probability of staying hot
	scalar q=1-(1-scalar(p))*(1-scalar(pi))/scalar(pi) // q= 1-(1-p)*ODDSAGAINSTBEINGHOT
	
	clear
	set obs `nshots'	
	gen rand1=runiform()
	gen period=_n
	gen state=(rand1<=scalar(pi))  if _n==1  //begin according to stationary distribution
	replace state=(rand1>scalar(p) & state[_n-1]==0)+ (rand1<=scalar(q) & state[_n-1]==1)
	*sum state

	gen rand2=runiform()
	gen make=(rand2<=scalar(fgn)&state==0| rand2<=scalar(fgh) & state==1)
	gen sid=1
	gen session=1

	gen dMade3ormore2=(make[_n-1]==1&make[_n-2]==1&make[_n-3]==1) if _n>3
	gen dMissed3ormore2=(make[_n-1]==0&make[_n-2]==0&make[_n-3]==0) if _n>3
end



*--------------------------------------------------
****	Positive Feedback Model
*
* Dumb 3+ Positive Feedback Model
* Everytime you hit 3+, your hit rate improves (could add with prob)
*You don't need a power simulation to see that it is underpowered, it is a pure issue of sample size
cap program drop genshots_3boost
program define genshots_3boost
	syntax [, nshots(integer 100) fgn(real .5) dfg(real .10) ]
clear
	set obs `nshots'
	gen rand1=runiform()
	scalar fgn=`fgn'
	scalar dfg=`dfg'
	scalar fghot=scalar(fgn)+scalar(dfg)
	gen period=_n
	gen make=(rand1<=scalar(fgn)) if period<4
	replace make =(rand1<=scalar(fgn)& !(make[_n-1]==1&make[_n-2]==1&make[_n-3]==1) |rand1<=scalar(fghot)&make[_n-1]==1&make[_n-2]==1&make[_n-3]==1)

	gen sid=1
	gen session=1

	gen dMade3ormore2=(make[_n-1]==1&make[_n-2]==1&make[_n-3]==1) if _n>3
	gen dMissed3ormore2=(make[_n-1]==0&make[_n-2]==0&make[_n-3]==0) if _n>3
end

*Average to 60
*.58  d=.1
*.55  d=.2
*.515 d=.3
*.47  d=.4

*Average to 55
*.53  d=.1
*.51  d=.2
*.48  d=.3
*.445  d=.4

*Average to 50
*.49  d=.1
*.47  d=.2
*.45  d=.3
*.42  d=.4

*Average to 45
*.44  d=.1
*.43  d=.2
*.41  d=.3
*.39  d=.4

*Average to 40
*.395  d=.1
*.385  d=.2
*.37  d=.3
*.36  d=.4

*genshots_3boost, nshots(50000) fgn(.58) dfg(.1)
*summ make


*===================streak feedback=======

cap program drop genshots_streakfeedback
program define genshots_streakfeedback
	syntax [, nshots(integer 100) fgn(real .5) dfg(real .10) ]
clear
	set obs `nshots'
	gen rand1=runiform()
	scalar fgn=`fgn'
	scalar dfg=`dfg'
	scalar fgh=scalar(fgn)+scalar(dfg)/2
	scalar fgc=scalar(fgn)-scalar(dfg)/2
	gen period=_n
	gen make=(rand1<=scalar(fgn)) if period<4
	replace make =(rand1<=scalar(fgc)&make[_n-1]==0&make[_n-2]==0&make[_n-3]==0| rand1<=scalar(fgn)& !((make[_n-1]==1&make[_n-2]==1&make[_n-3]==1)|(make[_n-1]==0&make[_n-2]==0&make[_n-3]==0)) |rand1<=scalar(fgh)&make[_n-1]==1&make[_n-2]==1&make[_n-3]==1)

	gen sid=1
	gen session=1

	gen dMade3ormore2=(make[_n-1]==1&make[_n-2]==1&make[_n-3]==1) if _n>3
	gen dMissed3ormore2=(make[_n-1]==0&make[_n-2]==0&make[_n-3]==0) if _n>3
end


	
*Begin program
cap program drop calc_diff
program define calc_diff, rclass
	syntax , command(string) method(string) [nshots(integer 300)]
	`command'
		disp "line 278"
	count if dMade3ormore ==1
	local NH3=r(N)
	count if dMissed3ormore ==1
	local NM3=r(N)
	if (`NH3'>=1 & `NM3'>=1){
	
			if "`method'"=="collapse"{
			*preserve
				keep if dMade3ormore ==1| dMissed3ormore==1
				collapse make,by(dMade3ormore dMissed3ormore )
				disp make[2]-make[1]
				return scalar diff=make[2]-make[1]
			*	restore
			
			}
			else{
	
			prtest make if dMade3ormore ==1| dMissed3ormore==1,by(dMade3ormore) // with a two-state model you are almost certainly in state 0 if you miss 3 or more in a row
			return scalar diff=r(P_2)-r(P_1)
			}
			/*
	
			*/
			
	}
	else{
			return scalar diff=.
	}
	*	
end


*****************************************
*Regime Switching Model:
*****************************************


cap program drop simregime
program define simregime
 syntax [, fg(string) reps(integer 10)]
		local dfgs = "10 20 30 40"
		local frachots= "10 15 20"

		*for some reason
		* `fracthot' == 10 didn't work
		* "`fracthot'" == "10" din't work, and all variations, only the following worked
			foreach dfg of local dfgs{
					foreach frachot of local frachots{
							display `frachot'
							if (`frachot' == 10) {
								local stayns = "98 99" // We can't go too low or probstayhot is to low, this give .82 and .91 respectively
							}
							else if (`frachot'== 15){
								local stayns = "97 99" // We can't go too low or probstayhot is to low, this give .83 and .94 respectively
							
							}
							else if (`frachot'== 20) {
								local stayns = "95 98" // We can't go too low or probstayhot is to low, this give .8 and .92 respectively
							}
							else {
								display "ERROR: Control flow issues"
								exit
							
							}
							foreach stayn of local stayns{
								disp "BiasAdjRegimeShift_fg`fg'-dfg`dfg'-stayn`stayn'-frachot`frachot'.dta"
								simulate  diff=r(diff) ///
								, reps(`reps') saving(BiasAdjRegimeShift_fg`fg'-dfg`dfg'-stayn`stayn'-frachot`frachot'.dta, replace) : calc_diff, command(genshots_regimeswitch, staynormal(.`stayn') frachot(.`frachot') nshots(100) fg(.`fg') dfg(.`dfg') )	method(collapse)																											
							}
					}
			}
	
end




*Average to 60
*.58  d=.1
*.55  d=.2
*.515 d=.3
*.47  d=.4

*Average to 55
*.53  d=.1
*.51  d=.2
*.48  d=.3
*.445  d=.4

*Average to 50
*.49  d=.1
*.47  d=.2
*.45  d=.3
*.42  d=.4

*Average to 45
*.44  d=.1
*.43  d=.2
*.41  d=.3
*.39  d=.4

*Average to 40
*.395  d=.1
*.385  d=.2
*.37  d=.3
*.36  d=.4



*genshots_3boost, nshots(50000) fgn(.58) dfg(.1)
*summ make


cap program drop sim3boost
program define sim3boost
 syntax [, reps(integer 10)]
		local dfgs = "10 20 30 40"
		local fgs = "40 45 50 55 60"
		matrix matfgn=(.395, .385, .37, .36\ .44, .43, .41, .39\ .48, .47, .45, .42\ .53, .51, .48, .445\ .58, .55, .515, .47)
		
		
			local row=0
			foreach fg of local fgs{
				local row = `row'+1
				local col=0
					foreach dfg of local dfgs{
						 local col=`col'+1
							local fgn=matfgn[`row',`col']
							display "fg = `fg'" ", fgn = `fgn'" ", dfg=`dfg'"
								disp "BiasAdjRegimeShift_fg`fg'-dfg`dfg'-stayn`stayn'-frachot`frachot'.dta"
								simulate  diff=r(diff) ///
								, reps(`reps') saving(BiasAdj3boost_fg`fg'-dfg`dfg'.dta, replace) : calc_diff, command(genshots_3boost, nshots(100) fgn(`fgn') dfg(.`dfg') )	method(collapse)																											
							}
				}
	
end
*sim3boost, reps(100)

*simregime, fg(45) reps(10)



cap program drop simstreakfeedback
program define simstreakfeedback
 syntax [, reps(integer 10)]
		local fgs = "40 45 50 55 60"
		local dfgs = "10 20 30 40"
			foreach fg of local fgs{
				foreach dfg of local dfgs{
				display "fg = `fg'" ", fgn = `fgn'" ", dfg=`dfg'"
					disp "BiasAdjStreakFeedback_fg`fg'-dfg`dfg'-stayn`stayn'-frachot`frachot'.dta"
					simulate  diff=r(diff) ///
					, reps(`reps') saving(BiasStreakFeedback_fg`fg'-dfg`dfg'.dta, replace) : calc_diff, command(genshots_streakfeedback, nshots(100) fgn(.`fg') dfg(.`dfg') )	method(collapse)																											
						
				}
				}
	
end




cap program drop simbernoulli
program define simbernoulli
 syntax [, reps(integer 10)]
		local fgs = "40 45 50 55 60"
			foreach fg of local fgs{
						local fgn =`fg'
						local dfg =0
				display "fg = `fg'" ", fgn = `fgn'" ", dfg=`dfg'"
					disp "BiasAdjRegimeShift_fg`fg'-dfg`dfg'-stayn`stayn'-frachot`frachot'.dta"
					simulate  diff=r(diff) ///
					, reps(`reps') saving(BiasAdjBernoulli_fg`fg'-dfg`dfg'.dta, replace) : calc_diff, command(genshots_3boost, nshots(100) fgn(.`fgn') dfg(.`dfg') )	method(collapse)																											
						
				}
	
end



